/*********************************************************************
*                      SEGGER Microcontroller GmbH                   *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*      (c) 2018  SEGGER Microcontroller GmbH                         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
-------------------------- END-OF-HEADER -----------------------------

File        : CTG_ConfigIO.c
Purpose     : External interface configuration for emCompress-ToGo.

*/

/*********************************************************************
*
*       #include Section
*
**********************************************************************
*/

#include "CTG.h"
#if defined(_WIN32) && _WIN32 > 0
  #include <stdio.h>
  #include <stdlib.h>
#endif

/*********************************************************************
*
*       Public code
*
**********************************************************************
*/

/*********************************************************************
*
*       CTG_Panic()
*
*  Function description
*    emCompress-ToGo failure handler.
*
*  Parameters
*    sFilename  - Filename where assertion failed.
*    LineNumber - Line number where assertion failed.
*    sAssertion - Assertion that failed.
*/
void CTG_Panic(const char *sFilename, int LineNumber, const char *sAssertion) {
#if defined(_WIN32) && _WIN32 > 0
  printf("CTG: assertion failure: %s at %s:%d\n", sAssertion, sFilename, LineNumber);
  exit(100);
#else
  CTG_USE_PARA(sFilename);
  CTG_USE_PARA(LineNumber);
  CTG_USE_PARA(sAssertion);
  for (;;) {
    /* Hang here */
  }
#endif
}

/*************************** End of file ****************************/
