/*********************************************************************
*                      SEGGER Microcontroller GmbH                   *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*      (c) 2018  SEGGER Microcontroller GmbH                         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
----------------------------------------------------------------------
File    : USB_Printer.c
Purpose : Sample implementation of USB printer device class
----------Literature--------------------------------------------------
Universal Serial Bus Device Class Definition for Printing Devices
Version 1.1 January 2000
--------  END-OF-HEADER  ---------------------------------------------
*/

#include <stdio.h>
#include <string.h>
#include "USB_PrinterClass.h"
#include "BSP.h"

/*********************************************************************
*
*       Static const data
*
**********************************************************************
*/
//
//  Information that is used during enumeration.
//
static const USB_DEVICE_INFO _DeviceInfo = {
  0x8765,                 // VendorId
  0x2114,                 // ProductId. Should be unique for this sample
  "Vendor",               // VendorName
  "Printer",              // ProductName
  "12345678901234567890"  // SerialNumber
};

/*********************************************************************
*
*       static data
*
**********************************************************************
*/
static U8 _acData[64 + 1]; // +1 for the terminating zero character

/*********************************************************************
*
*       static code
*
**********************************************************************
*/

/*********************************************************************
*
*       _GetDeviceIdString
*
*/
static const char * _GetDeviceIdString(void) {
  const char * s = "CLASS:PRINTER;"
                   "MODEL:HP LaserJet 6MP;"
                   "MANUFACTURER:Hewlett-Packard;"
                   "DESCRIPTION:Hewlett-Packard LaserJet 6MP Printer;"
                   "COMMAND SET:PJL,MLC,PCLXL,PCL,POSTSCRIPT;";
  return s;
}
/*********************************************************************
*
*       _GetHasNoError
*
*/
static U8 _GetHasNoError(void) {
  return 1;
}
/*********************************************************************
*
*       _GetIsSelected
*
*/
static U8 _GetIsSelected(void) {
  return 1;
}

/*********************************************************************
*
*       _GetIsPaperEmpty
*
*/
static U8 _GetIsPaperEmpty(void) {
  return 0;
}

/*********************************************************************
*
*       _OnDataReceived
*
*/
static int _OnDataReceived(const U8 * pData, unsigned NumBytes) {
  while (NumBytes > sizeof(_acData) - 1) {
    memcpy(_acData, pData, sizeof(_acData) - 1);
    _acData[sizeof(_acData) - 1] = 0;
    printf("%s", _acData);
    pData    += sizeof(_acData) - 1;
    NumBytes -= sizeof(_acData) - 1;
  }
  memcpy(_acData, pData, NumBytes);
  _acData[NumBytes] = 0;
  printf("%s", _acData);
  return 0;
}

/*********************************************************************
*
*       _OnReset
*
*/
static void _OnReset(void) {

}

static USB_PRINTER_API _PrinterAPI = {
  _GetDeviceIdString,
  _OnDataReceived,
  _GetHasNoError,
  _GetIsSelected,
  _GetIsPaperEmpty,
  _OnReset
};

/*********************************************************************
*
*       Public code
*
**********************************************************************
*/

/*********************************************************************
*
*       MainTask
*
* Function description
*   USB handling task.
*   Modify to implement the desired protocol
*/
#ifdef __cplusplus
extern "C" {     /* Make sure we have C-declarations in C++ programs */
#endif
void MainTask(void);
#ifdef __cplusplus
}
#endif
void MainTask(void) {
  USBD_Init();
  USBD_SetDeviceInfo(&_DeviceInfo);
  USB_PRINTER_Init(&_PrinterAPI);

#ifdef HANDLE_RECEIVE_DATA_IN_INTERRUPT

  USB_PRINTER_ConfigIRQProcessing();
  USBD_Start();
  for (;;) {
    USB_OS_Delay(1000);     // do nothing, everything is handled in interrupt.
  }

#else

  USBD_Start();
  while (1) {
    //
    // Wait for configuration
    //
    while ((USBD_GetState() & (USB_STAT_CONFIGURED | USB_STAT_SUSPENDED)) != USB_STAT_CONFIGURED) {
      BSP_ToggleLED(0);
      USB_OS_Delay(50);
    }
    BSP_SetLED(0);
    //
    // Use USB_PRINTER_Task() to receive data.
    //
    USB_PRINTER_Task();
  }

#endif
}




/**************************** end of file ***************************/

