/*********************************************************************
*                      SEGGER Microcontroller GmbH                   *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*      (c) 2018  SEGGER Microcontroller GmbH                         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
----------------------------------------------------------------------
File        : GUIDEMO_Intro.c
Purpose     : Introduction for emWin generic demo
              (This is also a good file to demo and explain basic
              emWin features by setting breakpoints)
----------------------------------------------------------------------
*/

#include <string.h>

#include "GUIDEMO.h"
#ifndef _WINDOWS
  #include "IP.h"
#endif

/*********************************************************************
*
*       Defines
*/
#define SCREEN_DIV          6  // 2^6 = 64

#define FACTOR_EMWIN        4
#define FACTOR_DESC        11
#define FACTOR_ANY_COMP    22
#define FACTOR_VERSION     31
#define FACTOR_LOGO        38
#define FACTOR_WWW         56

#define DIST_ANY_COMP      18

#define DELAY_GET_IP       80
#define DELAY_SHOW_NC      60

/*********************************************************************
*
*       GUIDEMO_Intro
*
**********************************************************************
*/
void GUIDEMO_Intro(void) {
  unsigned    xCenter;
  unsigned    yBase = 45 ;
  unsigned    xSize;
  char        acVersion[30] = "Version of emWin: ";
  static int  _notConnected = 0;
  static char _acText[50];
  static char _acIP[16];
#ifndef _WINDOWS
  int         IFaceId;
  int         IFaceIdOld = -1;
  U32         ipAddr;
  int         i;
#endif

  LCD_ControlCache(LCD_CC_LOCK);
  xSize   = LCD_GetXSize();
//  ySize   = LCD_GetYSize();
  xCenter = xSize / 2;
  GUIDEMO_DrawBkPic();
  GUI_SetTextMode(GUI_TM_NORMAL);
  //
  // emWin
  //
  GUI_SetColor(GUI_WHITE);
  GUI_SetFont(&GUI_FontRounded16);
  GUI_DispStringHCenterAt("emWin",                          120,     15);
  //
  // emWin description
  //
  GUI_SetFont(&GUI_Font6x8);
  GUI_DispStringHCenterAt("Universal graphic software",     xCenter, yBase + 0 );
  GUI_DispStringHCenterAt("for embedded applications",      xCenter, yBase + 10 );
  //
  // Any text
  //
  GUI_SetColor(0x88aaff);
  GUI_DispStringHCenterAt("Any CPU - Any Display",          xCenter, yBase + 22);
  GUI_DispStringHCenterAt("Any Application",                xCenter, yBase + 32);
  //
  // Compiled
  //
  GUI_SetColor(GUI_WHITE);
  GUI_SetFont(&GUI_Font10S_ASCII);
  GUI_DispStringHCenterAt("Compiled " __DATE__ " "__TIME__, xCenter, yBase + 44);
  //
  // Version
  //
  strcat(acVersion, GUI_GetVersionString());
  GUI_DispStringHCenterAt(acVersion,                        xCenter, yBase + 54);
  LCD_ControlCache(LCD_CC_UNLOCK);
  GUI_SetFont(&GUI_Font8x16);
#ifndef _WINDOWS
  for ( i = 0; i < DELAY_GET_IP ; i++ ){
    // Get the last registered interface ID as this is most likely the interface we want to use in this sample.
    IFaceId = IP_INFO_GetNumInterfaces() - 1;
    if ( IFaceIdOld != IP_IFaceIsReadyEx(IFaceId) || i == DELAY_SHOW_NC ) {
      IFaceIdOld = IP_IFaceIsReadyEx(IFaceId);
      GUI_ClearRect(1,109,158,124);
      if (IP_IFaceIsReadyEx(IFaceId) == 0) {
        if ( i >= DELAY_SHOW_NC || _notConnected == 1 ){
          strcpy(_acText, "IP: Not connected");
          _notConnected = 1;
        } else {
          strcpy(_acText, "IP: wait");
        }
      } else {
        ipAddr = IP_GetIPAddr(IFaceId);
        IP_PrintIPAddr(_acIP, ipAddr, sizeof(_acIP));
        strcpy(_acText,"IP: ");
        strcat(_acText, _acIP);
      }
      GUI_DispStringHCenterAt(_acText,                       xCenter, yBase + 65);
    }
    GUIDEMO_Delay(100);
  }
#endif
}

/*************************** End of file ****************************/
