/*********************************************************************
*                      SEGGER Microcontroller GmbH                   *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*      (c) 2018  SEGGER Microcontroller GmbH                         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
----------------------------------------------------------------------
File    : BSP_USB.c
Purpose : BSP for SEGGER emPower V2 with Freescale K66 MCU
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP_USB.h"
#include "SEGGER.h"
#include "RTOS.h"
#include "MK66F18.h"    // Device specific header file, contains CMSIS

/*********************************************************************
*
*       Defines, fixed
*
**********************************************************************
*/
#define USB_FRACTIONAL_VALUE          (0x02)

/*********************************************************************
*
*       Typedefs
*
**********************************************************************
*/
typedef void USB_ISR_HANDLER  (void);

/*********************************************************************
*
*       Prototypes
*
**********************************************************************
*/
#ifdef __cplusplus
extern "C" {
#endif
void USBHS_IRQHandler(void);
void USB0_IRQHandler(void);
#ifdef __cplusplus
}
#endif

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/
static USB_ISR_HANDLER * _pfOTG_FSHandler;
static USB_ISR_HANDLER * _pfOTG_HSHandler;

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/
/*********************************************************************
*
*       USB0_IRQHandler
*/
void USB0_IRQHandler(void) {
  OS_EnterInterrupt();
  if (_pfOTG_FSHandler) {
    (_pfOTG_FSHandler)();
  }
  OS_LeaveInterrupt();
}

/*********************************************************************
*
*       USBHS_IRQHandler
*/
void USBHS_IRQHandler(void) {
  OS_EnterInterrupt();
  if (_pfOTG_HSHandler) {
    (_pfOTG_HSHandler)();
  }
  OS_LeaveInterrupt();
}

/*********************************************************************
*
*       BSP_USB_InstallISR_Ex
*/
void BSP_USB_InstallISR_Ex(int ISRIndex, void (*pfISR)(void), int Prio) {
  IRQn_Type IRQSrc = (IRQn_Type)ISRIndex;

  (void)(Prio);  // Prio parameter is not used as the driver does not provide a CMSIS compatible IRQ prio
  if (IRQSrc == USB0_IRQn) {
    _pfOTG_FSHandler = pfISR;
  }
  if (IRQSrc == USBHS_IRQn) {
    _pfOTG_HSHandler = pfISR;
  }
  NVIC_SetPriority(IRQSrc, (1 << __NVIC_PRIO_BITS) - 2);
  NVIC_EnableIRQ(IRQSrc);
}

/*********************************************************************
*
*       BSP_USBH_InstallISR_Ex
*/
void BSP_USBH_InstallISR_Ex(int ISRIndex, void (*pfISR)(void), int Prio) {
  IRQn_Type IRQSrc = (IRQn_Type)ISRIndex;

  (void)(Prio);  // Prio parameter is not used as the driver does not provide a CMSIS compatible IRQ prio
  if (IRQSrc == USB0_IRQn) {
    _pfOTG_FSHandler = pfISR;
  }
  if (IRQSrc == USBHS_IRQn) {
    _pfOTG_HSHandler = pfISR;
  }
  NVIC_SetPriority(IRQSrc, (1 << __NVIC_PRIO_BITS) - 2);
  NVIC_EnableIRQ(IRQSrc);
}

/*********************************************************************
*
*       BSP_USB_Init
*/
void BSP_USB_Init(void) {
  SIM_SOPT1   |=  (1UL << SIM_SOPT1_USBREGEN_SHIFT);
  SIM_SOPT1   &= ~(1UL << SIM_SOPT1_USBSSTBY_SHIFT);
  /* MPU Configuration */
  MPU_CESR     = 0;                               // MPU is disabled. All accesses from all bus masters are allowed
  /* SIM Configuration */
  SIM_SOPT2   |= 1 << SIM_SOPT2_PLLFLLSEL_SHIFT;  // Select PLL output
  SIM_SOPT2   |= 1 << SIM_SOPT2_USBSRC_SHIFT;     // PLL/FLL selected as CLK source
  SIM_CLKDIV2 |= USB_FRACTIONAL_VALUE;            // USB Freq Divider
  SIM_SCGC4   |= SIM_SCGC4_USBOTG_MASK;           // USB Clock Gating
}

/****** End Of File *************************************************/
